package blatt4;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.TreeSet;

/**Beispiel-Implementierung der Breitensuche
 * @author rguderlei
 *
 */
public class BFSDemo {
	
	//Graph als Adjazenzliste.
	
	private Map<Node<Integer>, Set<Node<Integer>>> graph = new HashMap<Node<Integer>, Set<Node<Integer>>>();
	
	/**
	 * Konstruktor, baut den Beispielgraphen auf
	 */
	public BFSDemo(){
		//Erzeugung der Knoten, Der Inhalt eines Knotens ist dessen Nummer
		Node<Integer> a = new Node<Integer>(0);
		Node<Integer> b = new Node<Integer>(1);
		Node<Integer> c = new Node<Integer>(2);
		Node<Integer> d = new Node<Integer>(3);
		Node<Integer> e = new Node<Integer>(4);
		Node<Integer> f = new Node<Integer>(5);
		
		/*
		 * Beispiel-Graph:
		 *   b--d
		 *  /|  |\
		 * a |  | f
		 *  \|  |/
		 *   c--e
		 */
		//Adjazenzlisten fuer jeden Knoten erstellen
		Set<Node<Integer>> s = new TreeSet<Node<Integer>>();
		s.add(b); 
		s.add(c);
		graph.put(a,s);
		
		s = new TreeSet<Node<Integer>>();
		s.add(a); s.add(c); s.add(d);
		graph.put(b,s);
		
		s = new TreeSet<Node<Integer>>();
		s.add(a); s.add(b); s.add(e);
		graph.put(c,s);
		
		s = new TreeSet<Node<Integer>>();
		s.add(b); s.add(e); s.add(f);
		graph.put(d,s);
		
		s = new TreeSet<Node<Integer>>();
		s.add(f); s.add(c); s.add(d);
		graph.put(e,s);
		
		s = new TreeSet<Node<Integer>>();
		s.add(d); s.add(e);
		graph.put(f,s);
	}
	
    /**Breitensuche.
     * @param start Start-Knoten
     * @param ziel End-Knoten
     * @return Laenge des kuerzesten Pfades von start nach ziel
     */
    public int bfs(Node<Integer> start, Node<Integer> ziel) {
	        Queue<Node<Integer>> toBeVisited = new LinkedList<Node<Integer>>();
	        Set<Node<Integer>> visited = new TreeSet<Node<Integer>>();
	
	        toBeVisited.add(start);
	        while (!toBeVisited.isEmpty()) {
	                Node<Integer> node = toBeVisited.poll();
	                if (node.equals(ziel))
	                        return node.dist;
	                visited.add(node);
	                for (Node<Integer> neighbor : getNeighbors(node)){
	                        if (!visited.contains(neighbor))
	                                toBeVisited.add(new Node<Integer> (neighbor.value, node.dist+1));
	                }
	        }
	
	        return -1; // nicht erreichbar
	}
    

	private Set<Node<Integer>> getNeighbors(Node<Integer> node) {
		return graph.get(node);
	}
	
	public static void main(String[] args){
		BFSDemo d = new BFSDemo();
		System.out.println(d.bfs(new Node<Integer>(0), new Node<Integer>(5)));
	}
}
