package blatt4;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.TreeSet;

/**Diese Klasse implementiert einen Lösungsvorschlag basierend auf dem zur Verfügung gestellten
 * Beispiel.
 * @author rguderlei
 *
 */
public class ErdoesNumberExample {
	
	//Graph als Adjazenzliste.
	private Map<Node<String>, Set<Node<String>>> graph = new HashMap<Node<String>, Set<Node<String>>>();
	//Mapping Nachname -> voller Name
	private Map<String, String> fullName = new HashMap<String, String>();
	
	/**
	 * Konstruktor.
	 */
	public ErdoesNumberExample(){
	}
	
	/**Erzeugt aus einer Liste von Autoren die nötigen Einträge für den Graphen.
	 * In der Datei sind die Nachnamen eindeutig, deshalb werden diese als Key der Adjazenz-Liste verwendet.
	 * @param authors String - Array der Autoren
	 */
	public void addCombination(String[] authors){
		if(authors.length>1){
			for(int i=0; i<authors.length; i++){
				Node<String> author = new Node<String>(ReferenceParser.getName(authors[i]));
				for(int j=0; j<authors.length; j++){
					Node<String> coauthor = new Node<String>(ReferenceParser.getName(authors[j]));
					if(i!=j){
						if(graph.keySet().contains(author)){
							Set<Node<String>> set = graph.get(author);
							set.add(coauthor);
							graph.put(author, set);
						} else {
							Set<Node<String>> set = new HashSet<Node<String>>();
							set.add(coauthor);
							graph.put(author, set);
						}
						if(!fullName.keySet().contains(author.value)){
							fullName.put(author.value, authors[i]);
						}
						if(!fullName.keySet().contains(coauthor.value)){
							fullName.put(coauthor.value, authors[j]);
						}
					}
				}
			}
		}
	}
	
	
    /**Breitensuche.
     * @param start Start-Knoten
     * @param ziel End-Knoten
     * @return Laenge des kuerzesten Pfades von start nach ziel
     */
    public int bfs(Node<String> start, Node<String> ziel) {
	        Queue<Node<String>> toBeVisited = new LinkedList<Node<String>>();
	        Set<Node<String>> visited = new TreeSet<Node<String>>();
	
	        toBeVisited.add(start);
	        while (!toBeVisited.isEmpty()) {
	                Node<String> node = toBeVisited.poll();
	                if (node.equals(ziel))
	                        return node.dist;
	                visited.add(node);
	                for (Node<String> neighbor : getNeighbors(node)){
	                        if (!visited.contains(neighbor))
	                                toBeVisited.add(new Node<String> (neighbor.value, node.dist+1));
	                }
	        }
	
	        return -1; // nicht erreichbar
	}
    
    /**
	 * Diese Methode gibt die Namen und Erdoes-Zahlen aus.
	 */
	public void print(){
		Set<Node<String>> keys = graph.keySet();
		
		for(Node<String> key : keys){
			System.out.println((String)fullName.get(key.value) + ": " + bfs(key, new Node<String>("Erdoes")));
		}
	}
    
    
	private Set<Node<String>> getNeighbors(Node<String> node) {
		return graph.get(node);
	}
	
	public static void main(String[] args){
		ErdoesNumberExample d = new ErdoesNumberExample();
		//System.out.println(d.bfs(new Node<String>(0), new Node<String>(5)));
	}
}
